%{ open Ast %}

%token SEMI LPAREN RPAREN LBRACE RBRACE COMMA
%token PLUS MINUS TIMES DIVIDE ASSIGN
%token EQ NEQ LT LEQ GT GEQ COMBINE
%token IF ELSE FOR
%token NUMBER TEXT
%token <string> ID
%token <string> LITERAL
%token <string> STR
%token EOF
%token FUNCTION SHAPE OBJECT
%token WORLD
%token DEREFERENCE
%token TOSCREEN

%nonassoc NOELSE
%nonassoc ELSE

%left ASSIGN
%left EQ NEQ
%left LT GT LEQ GEQ
%left COMBINE
%left PLUS MINUS
%left TIMES DIVIDE

%start program
%type <Ast.program> program

%%

program:
  /* nothing */        { [], [] }
  | program vdecl      { ($2 :: fst $1), snd $1 }
  | program fdecl      { fst $1, ($2 :: snd $1) }

fdecl:
   FUNCTION ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list RBRACE
     { { fname = $2;
	 formals = $4;
	 locals = List.rev $7;
	 body = List.rev $8 } }

formals_opt:
    /* nothing */ { [] }
  | formal_list   { List.rev $1 }

formal_list:
  | ID                       { [$1] }
  | formal_list COMMA ID     { $3 :: $1 }
    
vdecl_list:
  /* nothing */  { [] }
  | vdecl_list vdecl { $2 :: $1 }

vdecl:
  OBJECT ID LPAREN p_opt RPAREN SEMI { { 
    vartype = "Object"; varname = $2;  varparams = $4;} } 
  | SHAPE ID LPAREN p_opt RPAREN SEMI { { 
    vartype = "Shape"; varname = $2;  varparams = $4;} } 
  | WORLD LPAREN p_opt RPAREN SEMI { { 
    vartype = "World"; varname = "World";  varparams = $3;} } 
              
p_opt:
  /* nothing */                 { [] }
  | p_list                      { $1 }

p_list:
  param                         { [$1] }
  | p_list COMMA param          { $3 :: $1 }

param:
  NUMBER ID ASSIGN LITERAL    { { key   = $2;
    value = $4;} }
  | TEXT ID ASSIGN STR    { {   key   = $2;
    value = String.sub $4 1 ((String.length $4)-2) ;} }
  | NUMBER ID     { {   key   = $2;
    value = "0";} }
  | TEXT ID     { {   key   = $2;
    value = "";} }
                       

stmt_list:
    /* nothing */  { [] }
  | stmt_list stmt { $2 :: $1 }

stmt:
  expr SEMI { Expr($1) }
  | LBRACE stmt_list RBRACE { Block(List.rev $2) }
  | IF LPAREN objectexpr RPAREN stmt %prec NOELSE { If($3, $5, Block([])) }
  | IF LPAREN objectexpr RPAREN stmt ELSE stmt    { If($3, $5, $7) }
  | FOR LPAREN objectexpr SEMI objectexpr SEMI objectexpr RPAREN stmt
     { For($3, $5, $7, $9) }
  | TOSCREEN objectexpr SEMI      { ToScreen($2) }
     
expr_opt:
    /* nothing */ { Noexpr }
  | objectexpr          { $1 }

expr:
  ID                                        { Id($1) }
  | ID DEREFERENCE ID ASSIGN objectexpr     { Assign($1, $3, $5) }   
  | ID LPAREN actuals_opt RPAREN            { Call($1, $3) }
  | LPAREN expr RPAREN                      { $2 }

objectexpr:
  LITERAL                                     { Literal($1) }
  | STR                                       { Str($1) } 
  | objectexpr PLUS objectexpr                { Binop($1, Add, $3) }
  | objectexpr MINUS objectexpr               { Binop($1, Sub, $3) }
  | objectexpr TIMES objectexpr               { Binop($1, Mult, $3) }
  | objectexpr DIVIDE objectexpr              { Binop($1, Div, $3) }
  | objectexpr EQ objectexpr                  { Binop($1, Equal, $3) }
  | objectexpr NEQ objectexpr                 { Binop($1, Neq, $3) }
  | objectexpr LT objectexpr                  { Binop($1, Less, $3) }
  | objectexpr LEQ objectexpr                 { Binop($1, Leq, $3) }
  | objectexpr GT objectexpr                  { Binop($1, Greater, $3) }
  | objectexpr GEQ objectexpr                 { Binop($1, Geq, $3) }
  | objectexpr COMBINE objectexpr             { Binop($1, Combine,  $3) }
  | ID DEREFERENCE ID ASSIGN objectexpr       { AssignToObject($1, $3, $5) } 
  | ID DEREFERENCE ID                         { Access($1, $3) }
  | LPAREN objectexpr RPAREN                  { $2 }
     
actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }
