{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "//"     { comment lexbuf }           (* Comments *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| ';'      { SEMI }
| ','      { COMMA }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| ":="     { ASSIGN }
| "@"     { DEREFERENCE }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "++"     { COMBINE }
| "if"     { IF }
| "else"   { ELSE }
| "for"    { FOR }
| "function" { FUNCTION }
| "shape"  { SHAPE }
| "object" { OBJECT }
| "number" { NUMBER }
| "text"   { TEXT }
| "toscreen"  { TOSCREEN }
| "world" { WORLD }

| ['0'-'9']+ as lxm { LITERAL(lxm) }  (* Integers *)
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }  (* Identifiers *)
| '\"' [^ '\"']* '\"' as lxm { STR(lxm) }  (* Strings *)
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }
| eof { EOF }

and comment = parse
  "//" { token lexbuf }
| _    { comment lexbuf }
